@@ -79,7 +79,7 @@ class SurfingEntrance(object):
 
         rq_license = SurfingConfigurator().get_license_settings('rq')
         raw_data_downloader = RawDataDownloader(rq_license)
-        return raw_data_downloader.download(yesterday, yesterday)
+        return raw_data_downloader.download(yesterday, yesterday), raw_data_downloader.updated_count
     
     def update_basic(self, yesterday=None):
         if not yesterday:
@@ -88,14 +88,15 @@ class SurfingEntrance(object):
         from .data.fund.basic.basic_data_processor import BasicDataProcessor
 
         basic_data_processor = BasicDataProcessor()
-        return basic_data_processor.process_all(yesterday, yesterday)
+        return basic_data_processor.process_all(yesterday, yesterday), basic_data_processor.updated_count
     
     def update_derived(self, yesterday=None):
         if not yesterday:
             yesterday = self._get_yesterday()
 
         from .data.fund.derived.derived_data_processor import DerivedDataProcessor
-        return DerivedDataProcessor().process_all(yesterday, yesterday)
+        derived_data_processor = DerivedDataProcessor()
+        return derived_data_processor.process_all(yesterday, yesterday), derived_data_processor.updated_count
 
     def update_view(self, yesterday=None):
         if not yesterday:
@@ -110,23 +111,31 @@ class SurfingEntrance(object):
         yesterday = self._get_yesterday()
 
         failed_tasks = {}
+        updated_count = {}
 
         print('Step 1. Start raw data download')
-        raw_failed_tasks = self.download_raw(yesterday)
+        raw_failed_tasks, raw_updated_count = self.download_raw(yesterday)
+        if 'em_tradedates' in raw_failed_tasks:
+            print('Step 1. No raw data to download')
+            print(f'Done data update, {yesterday} is not trading day')
+            return
         if len(raw_failed_tasks) > 0:
             failed_tasks['raw'] = raw_failed_tasks
+        updated_count['raw'] = raw_updated_count
         print('Step 1. Done raw data download')
 
         print('Step 2. Start update basic')
-        basic_failed_tasks = self.update_basic(yesterday)
+        basic_failed_tasks, basic_updated_count = self.update_basic(yesterday)
         if len(basic_failed_tasks) > 0:
             failed_tasks['basic'] = basic_failed_tasks
+        updated_count['basic'] = basic_updated_count
         print('Step 2. Done update basic')
 
         print('Step 3. Start update derived')
-        derived_failed_tasks = self.update_derived(yesterday)
+        derived_failed_tasks, derived_updated_count = self.update_derived(yesterday)
         if len(derived_failed_tasks) > 0:
             failed_tasks['derived'] = derived_failed_tasks
+        updated_count['derived'] = derived_updated_count
         print('Step 3. Done update derived')
         
         print('Step 4. Start update view')
@@ -136,10 +145,12 @@ class SurfingEntrance(object):
         print('Step 4. Done update view')
 
         print('Done data update')
+        print(f'Failed tasks:\n{failed_tasks}')
+        print(f'Updated count:\n{updated_count}')
 
         from .util.wechat_bot import WechatBot
         wechat_bot = WechatBot()
-        wechat_bot.send_data_update_status(yesterday, failed_tasks)
+        wechat_bot.send_data_update_status(yesterday, failed_tasks, updated_count)
     
 
 if __name__ == "__main__":
