@@ -70,6 +70,25 @@ class BasicDataApi(object):
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))            
 
+    def get_fund_nav_with_date(self, start_date, end_date, fund_list):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                        FundNav.fund_id,
+                        FundNav.adjusted_net_value, 
+                        FundNav.datetime
+                    ).filter(
+                        FundNav.fund_id.in_(fund_list),
+                        FundNav.datetime >= start_date,
+                        FundNav.datetime <= end_date,
+                    )
+                    
+                tag_df = pd.read_sql(query.statement, query.session.bind)
+                return tag_df
+            
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))        
+                
     def get_stock_price(self, stock_list):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
@@ -120,6 +139,21 @@ class BasicDataApi(object):
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))            
 
+    def delete_index_price(self, index_id_list, start_date, end_date):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    IndexPrice
+                ).filter(
+                    IndexPrice.index_id.in_(index_id_list),
+                    IndexPrice.datetime >= start_date,
+                    IndexPrice.datetime <= end_date,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
 
     def get_fund_nav_adjusted_nv(self, fund_list, start_date, end_date):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -223,3 +257,15 @@ class BasicDataApi(object):
                 return None
                 
         return fund_id_mapping
+
+    def get_fund_size(self):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                        FundSize,
+                    )
+                tag_df = pd.read_sql(query.statement, query.session.bind)
+                return tag_df
+            
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
\ No newline at end of file
