@@ -125,6 +125,8 @@ class BasicDataPart1(object):
 
     def fund_nav(self, start_date, end_date):
         try:
+            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=3)
+            start_date = start_date.strftime('%Y%m%d')
             df = RawDataApi().get_em_fund_nav(start_date, end_date)
             df['fund_id'] = df.apply(
                 lambda x: self._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
@@ -138,7 +140,10 @@ class BasicDataPart1(object):
                     'UNITYIELD10K': 'daily_profit',
                     'YIELDOF7DAYS': 'weekly_yield'
                 })
+            df['change_rate'] = df.adjusted_net_value / df.adjusted_net_value.shift(1) - 1
             df['weekly_yield'] = df['weekly_yield'].map(lambda x: x / 100.0)
+            end_date = datetime.datetime.strptime(end_date, '%Y%m%d').date()
+            df = df[df.datetime == end_date]
             self._upload_basic(df, FundNav.__table__.name)
             return True
         except Exception as e:
@@ -148,6 +153,8 @@ class BasicDataPart1(object):
 
     def index_price(self, start_date, end_date):
         try:
+            start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=3)
+            start_date = start_date.strftime('%Y%m%d')
             cm_index_price = RawDataApi().get_raw_cm_index_price_df(start_date, end_date)
             cxindex_index_price = RawDataApi().get_cxindex_index_price_df(start_date, end_date)
             yahoo_index_price = RawDataApi().get_yahoo_index_price_df(start_date, end_date)
@@ -170,6 +177,7 @@ class BasicDataPart1(object):
                 df['volume'] = float('Nan')
                 df['total_turnover'] = float('Nan')
                 df['index_id'] = c
+                df['ret'] = df.close / df.close.shift(1) -1
                 df_list.append(df.copy())
 
             index_dic = {k: v for k, v in zip(
@@ -182,6 +190,7 @@ class BasicDataPart1(object):
                 if index_dic[index_i] == 'gold':
                     continue 
                 dftmp = rq_index_price[rq_index_price['order_book_id'] == index_i].copy()
+                dftmp['ret'] = dftmp.close / dftmp.close.shift(1) - 1
                 res.append(dftmp)
             rq_index_price = pd.concat(res)
             df_list.append(yahoo_index_price)
@@ -190,7 +199,8 @@ class BasicDataPart1(object):
 
             df = pd.concat(df_list).drop(['order_book_id'], axis=1)
             df['datetime'] = pd.to_datetime(df['datetime'])
-            df = df.drop(['ret'], axis=1)
+            end_date = datetime.datetime.strptime(end_date, '%Y%m%d').date()
+            df = df[df.datetime==end_date]
             self._upload_basic(df, IndexPrice.__table__.name)
             return True
         except Exception as e:
@@ -321,6 +331,7 @@ class BasicDataPart1(object):
     def em_index_price(self, start_date, end_date):
         try:
             start_date = datetime.datetime.strptime(start_date, '%Y%m%d') - datetime.timedelta(days=5)
+            del_date = start_date + datetime.timedelta(days=1)
             start_date = start_date.strftime('%Y%m%d')
             index_info = BasicDataApi().get_index_info()
             em_id_list = index_info['em_id'].dropna().tolist()
@@ -337,10 +348,13 @@ class BasicDataPart1(object):
                 df['open'] = float('nan')
                 df['total_turnover'] = float('nan')
                 df['volume'] = float('nan')
+                df['ret'] = df.close / df.close.shift(1) - 1
                 res.append(df)
-                index_id_list.append(index_id)
-            BasicDataApi().delete_index_price(index_id_list, start_date, end_date)
+                if not df.empty:
+                    index_id_list.append(index_id)
+            BasicDataApi().delete_index_price(index_id_list, del_date, end_date)
             df = pd.concat(res, axis=0, sort=False)
+            df = df[df.datetime >= del_date.date()]
             self._upload_basic(df, IndexPrice.__table__.name)
             return True
         except Exception as e:
