@@ -31,10 +31,23 @@ class RqRawDataDownloader(object):
         self._order_book_id_2_index_id = {order_book_id : index_id for order_book_id, index_id in 
             zip(index_df['order_book_id'], index_df['index_id']) if order_book_id != 'not_available'}
 
-    def _upload_raw(self, df, table_name, if_exists='append'):
+        self.updated_count = {}
+
+    def _upload_raw(self, df, table_name, to_truncate=False):
         print(table_name)
         # print(df)
-        df.to_sql(table_name, RawDatabaseConnector().get_engine(), index=False, if_exists=if_exists)
+        if to_truncate:
+            with RawDatabaseConnector().managed_session() as mn_session:
+                try:
+                    mn_session.execute(f'TRUNCATE TABLE {table_name}')
+                    mn_session.commit()
+                except Exception as e:
+                    print(f'Failed to truncate table {table_name} <err_msg> {e}')
+        df.to_sql(table_name, RawDatabaseConnector().get_engine(), index=False, if_exists='append')
+        
+        if table_name not in self.updated_count:
+            self.updated_count[table_name] = 0
+        self.updated_count[table_name] += df.shape[0]
 
     def _get_date_list(self, start_date, end_date):
         start_datetime = datetime.datetime.strptime(start_date, '%Y%m%d')
@@ -94,8 +107,7 @@ class RqRawDataDownloader(object):
                 })
             df = pd.DataFrame(instruments)
             if df is not None:
-                df['update_time'] = datetime.date.today()
-                self._upload_raw(df, RqFundSize.__table__.name, if_exists='replace')
+                self._upload_raw(df, RqFundSize.__table__.name, to_truncate=True)
             return True
         except Exception as e:
             print(e)
@@ -231,7 +243,7 @@ class RqRawDataDownloader(object):
                 })
             df = pd.DataFrame(order_book_ids)
             if df is not None:
-                self._upload_raw(df, StockInfo.__table__.name, if_exists='replace')
+                self._upload_raw(df, StockInfo.__table__.name, to_truncate=True)
             return True
         except Exception as e:
             print(e)
@@ -253,42 +265,54 @@ class RqRawDataDownloader(object):
 
     def download_all(self, start_date, end_date):
         failed_tasks = []
-        if not self.rq_fund_indicator(start_date, end_date):
-            failed_tasks.append('rq_fund_indicator')
+        # # Safe to disable
+        # if not self.rq_fund_indicator(start_date, end_date):
+        #     failed_tasks.append('rq_fund_indicator')
 
-        if not self.rq_fund_nav(start_date, end_date):
-            failed_tasks.append('rq_fund_nav')
+        # # Safe to disable
+        # if not self.rq_fund_nav(start_date, end_date):
+        #     failed_tasks.append('rq_fund_nav')
 
+        # # Safe to disable
+        # if not self.rq_index_weight(start_date, end_date):
+        #     failed_tasks.append('rq_index_weight')
+
+        # # Safe to disable
+        # if not self.rq_stock_fin_fac(start_date, end_date):
+        #     failed_tasks.append('rq_stock_fin_fac')
+
+        # # Safe to disable
+        # if not self.rq_stock_post_price(start_date, end_date):
+        #     failed_tasks.append('rq_stock_post_price')
+
+        # # Safe to disable
+        # if not self.rq_stock_price(start_date, end_date):
+        #     failed_tasks.append('rq_stock_price')
+
+        # # Safe to disable
+        # if not self.rq_stock_valuation(start_date, end_date):
+        #     failed_tasks.append('rq_stock_valuation')
+
+        # # Safe to disable
+        # if not self.stock_info():
+        #     failed_tasks.append('stock_info')
+
+        # # Safe to disable
+        # if not self.trading_day_list(start_date, end_date):
+        #     failed_tasks.append('trading_day_list')
+
+        # # Safe to disable
+        # if not self.rq_fund_size():
+        #     failed_tasks.append('rq_fund_size')
+            
+        # TODO: gold
         if not self.rq_index_price(start_date, end_date):
             failed_tasks.append('rq_index_price')
 
-        if not self.rq_index_weight(start_date, end_date):
-            failed_tasks.append('rq_index_weight')
-
-        if not self.rq_stock_fin_fac(start_date, end_date):
-            failed_tasks.append('rq_stock_fin_fac')
-
-        if not self.rq_stock_post_price(start_date, end_date):
-            failed_tasks.append('rq_stock_post_price')
-
-        if not self.rq_stock_price(start_date, end_date):
-            failed_tasks.append('rq_stock_price')
-
-        if not self.rq_stock_valuation(start_date, end_date):
-            failed_tasks.append('rq_stock_valuation')
-
-        if not self.trading_day_list(start_date, end_date):
-            failed_tasks.append('trading_day_list')
-
+        # TODO: used in basic db, replace to EM
         if not self.fund_rating(start_date, end_date):
             failed_tasks.append('fund_rating')
 
-        if not self.stock_info():
-            failed_tasks.append('stock_info')
-
-        if not self.rq_fund_size():
-            failed_tasks.append('rq_fund_size')
-        
         return failed_tasks
 
 if __name__ == "__main__":
