@@ -48,6 +48,7 @@ class FundDailyCollectionProcessor(object):
             except Exception as e:
                 print('Failed fill get data <err_msg> {}'.format(e))
 
+    # TODO: use data in basic
     def get_indicator_df(self):
         with RawDatabaseConnector().managed_session() as quant_session:
             try:
@@ -175,6 +176,12 @@ class FundDailyCollectionProcessor(object):
 
     def append_data(self, table_name, data_append_directly_data_df):
         if not data_append_directly_data_df.empty:
+            with ViewDatabaseConnector().managed_session() as mn_session:
+                try:
+                    mn_session.execute(f'TRUNCATE TABLE {table_name}')
+                    mn_session.commit()
+                except Exception as e:
+                    print(f'Failed to truncate table {table_name} <err_msg> {e}')
             data_append_directly_data_df.to_sql(table_name, ViewDatabaseConnector().get_engine(), index = False, if_exists = 'append')
             print('新数据已插入')
         else:
