@@ -455,8 +455,8 @@ class TaaTunerParam:
     PCT_PAIR = {
         'gem':'ps_ttm',
         'hs300':'pe_ttm',
-        'csi500':'pe_pct',
-        'sp500rmb':'pe_pct',
+        'csi500':'pe_ttm',
+        'sp500rmb':'pe_ttm',
     }
 
 if __name__ == '__main__':
