@@ -6,24 +6,26 @@ import pandas as pd
 import os
 import time
 
-from .asset_helper import SAAHelper, TAAHelper
+from .asset_helper import SAAHelper, TAAHelper, FAHelper
 from .trader import AssetTrader, FundTrader
 from .report import ReportHelper
 from ...data.manager.manager_fund import FundDataManager
 from ...data.struct import AssetWeight, AssetPrice, AssetPosition, AssetValue
-from ...data.struct import FundPosition, TAAParam, AssetTradeParam, FundTradeParam
+from ...data.struct import FundPosition, TAAParam, AssetTradeParam, FundTradeParam, FAParam
 
 class FundEngine:
 
     DEFAULT_CASH_VALUE = 1e8
 
-    def __init__(self, data_manager: FundDataManager, trader, taa_params:TAAParam=None):
+    def __init__(self, data_manager: FundDataManager, trader, taa_params:TAAParam=None, fa_params:FAParam=None, taa_param_details:dict=None):
         self._dm = data_manager
-        self._saa_helper = SAAHelper() # 战略目标
-        self._taa_helper = TAAHelper(taa_params=taa_params) if taa_params else None
+        self._saa_helper = SAAHelper()
+        self._taa_helper = TAAHelper(taa_params=taa_params, taa_param_details=taa_param_details) if taa_params or taa_param_details else None 
+        self._fa_helper = FAHelper(fa_params=fa_params) if fa_params else None
         self._report_helper = ReportHelper()
         self._trader = trader
         self._pending_trades = []
+        assert not self.is_fund_backtest or self._fa_helper, 'if this is a fund backtest, fa_helper is necessary'
 
     def init(self):
         if not self._dm.inited:
@@ -33,6 +35,16 @@ class FundEngine:
             self._taa_helper.init()
         self._report_helper.init()
         self._trader.init()
+        # preps
+        self._prep_fund_purchase_fees = None
+        self._prep_fund_redeem_fees = None
+        self._prep_asset_price = None
+        self._prep_fund_nav = None
+        self._prep_fund_unit_nav = None
+        self._prep_fund_score = None
+        self._prep_fund_score_raw = None
+        self._prep_target_asset_allocation = None
+        self._prep_target_fund_allocation = None
 
     def setup(self, saa: AssetWeight):
         # setup helpers
@@ -47,28 +59,38 @@ class FundEngine:
         return isinstance(self._trader, FundTrader)
 
     def prep_data(self, dt):
+        # basic
+        if not self._prep_fund_purchase_fees:
+            self._prep_fund_purchase_fees = self._dm.get_fund_purchase_fees()
+        if not self._prep_fund_redeem_fees:
+            self._prep_fund_redeem_fees = self._dm.get_fund_redeem_fees()
+        # DATA
         self._prep_asset_price = self._dm.get_index_price_data(dt)
-        if self.is_fund_backtest:
-            self._prep_fund_nav = self._dm.get_fund_nav(dt)
-            self._prep_fund_score, self._prep_fund_score_raw = self._dm.get_fund_scores(dt)
-        else:
-            self._prep_fund_nav = {}
-            self._prep_fund_score = {}
-            self._prep_fund_score_raw = {}
+        self._prep_fund_nav = self._dm.get_fund_nav(dt) if self.is_fund_backtest else {}
+        self._prep_fund_unit_nav = self._dm.get_fund_unit_nav(dt) if self.is_fund_backtest else {}
+        self._prep_fund_score, self._prep_fund_score_raw = self._dm.get_fund_scores(dt) if self.is_fund_backtest else ({}, {})
+        # ALLOCATION
         self._prep_target_asset_allocation = self.calc_asset_allocation(dt)
+        self._prep_target_fund_allocation = self.calc_fund_allocation(dt) if self.is_fund_backtest else None
 
     def calc_trade(self, dt, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None):
-        if self.is_fund_backtest:
+        v_asset_position, asset_trade_list = self._trader.calc_asset_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation)
+        if asset_trade_list and self.is_fund_backtest:
             assert cur_fund_position, 'cur_fund_position should not be None in fund backtest run'
-            virtual_position, trade_list = self._trader.calc_fund_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation, cur_fund_position, self._prep_fund_nav, self._prep_fund_score)
+            assert self._prep_target_fund_allocation, 'target fund allocation should be prepared already'
+            v_fund_position, fund_trade_list = self._trader.calc_fund_trade(
+                dt, self._prep_target_fund_allocation, cur_fund_position, self._prep_fund_nav, 
+                self._prep_fund_purchase_fees, self._prep_fund_redeem_fees
+            )
+            return fund_trade_list
         else:
-            virtual_position, trade_list = self._trader.calc_asset_trade(dt, cur_asset_position, self._prep_asset_price, self._prep_target_asset_allocation)
-        return trade_list
+            return asset_trade_list
 
     def finalize_trade(self, dt, trades: list, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None):
         if self.is_fund_backtest:
             assert cur_fund_position, 'cur_fund_position should not be None in fund backtest run'
-            self._pending_trades, traded_list = self._trader.finalize_trade(dt, trades, self._prep_asset_price, cur_asset_position, cur_fund_position, self._prep_fund_nav)
+            self._pending_trades, traded_list = self._trader.finalize_trade(dt, trades, self._prep_asset_price, cur_asset_position, cur_fund_position, self._prep_fund_nav, self._prep_fund_unit_nav,
+                self._prep_fund_purchase_fees, self._prep_fund_redeem_fees)
         else:
             self._pending_trades, traded_list = self._trader.finalize_trade(dt, trades, self._prep_asset_price, cur_asset_position)
         return traded_list 
@@ -77,20 +99,20 @@ class FundEngine:
         cur_asset_price = self._dm.get_index_price_data(dt)
         cur_saa = self._saa_helper.on_price(dt, cur_asset_price)
         if self._taa_helper:
-            asset_pct = self._dm.get_index_pcts(dt)
-            cur_taa = self._taa_helper.on_price(dt, cur_asset_price, cur_saa, asset_pct)
+            asset_pct = self._dm.get_index_pcts_df(dt)
+            cur_taa = self._taa_helper.on_price(dt, cur_asset_price, cur_saa, asset_pct, {})
         else:
             cur_taa = cur_saa
         return cur_taa
 
-    def update_reporter(self, dt, trade_list):
-        self._report_helper.update(dt, self.cur_asset_position.copy(), self._prep_asset_price, self._pending_trades, self.cur_fund_position.copy() if self.is_fund_backtest else None, self._prep_fund_nav, trade_list, self._prep_fund_score, self._prep_fund_score_raw, self._prep_target_asset_allocation)
+    def calc_fund_allocation(self, dt):
+        return self._fa_helper.on_price(dt, self._prep_target_asset_allocation, self._prep_fund_score)
+
+    def update_reporter(self, dt, trade_list, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None):
+        self._report_helper.update(dt, cur_asset_position, self._prep_asset_price, self._pending_trades, cur_fund_position if self.is_fund_backtest else None, self._prep_fund_nav, trade_list, self._prep_fund_score, self._prep_fund_score_raw, self._prep_target_asset_allocation)
 
 class FundBacktestEngine(FundEngine):
 
-    def __init__(self, data_manager: FundDataManager, trader, taa_params:TAAParam=None):
-        FundEngine.__init__(self, data_manager, trader, taa_params)
-
     def run(self, saa: AssetWeight, start_date: datetime.date=None, end_date: datetime.date=None, cash: float=None, print_time=False):
         cash = cash or self.DEFAULT_CASH_VALUE
         # init position
@@ -129,7 +151,7 @@ class FundBacktestEngine(FundEngine):
         # book trade
         self.book_trades(trade_list)
         # update
-        self.update_reporter(dt, traded_list)        
+        self.update_reporter(dt, traded_list, self.cur_asset_position, self.cur_fund_position)        
         _tm_finish = time.time()
 
         if print_time:
@@ -185,25 +207,28 @@ def fund_backtest_without_taa(m: FundDataManager, saa: AssetWeight):
     asset_param = AssetTradeParam() # type in here
     fund_param = FundTradeParam() # type in here
     t = FundTrader(asset_param, fund_param)
-    b = FundBacktestEngine(data_manager=m, trader=t, taa_params=None)
+    fa_param = FAParam() # type in here
+    b = FundBacktestEngine(data_manager=m, trader=t, taa_params=None, fa_params=fa_param)
     b.init()
     b.run(saa=saa)
 
 def fund_backtest(m: FundDataManager, saa: AssetWeight):
-    taa_param = TAAParam()  # type in here
     asset_param = AssetTradeParam() # type in here
-    fund_param = FundTradeParam() # type in here
+    fund_param = FundTradeParam(EnableCommission=True) # type in here
     t = FundTrader(asset_param, fund_param)
-    b = FundBacktestEngine(data_manager=m, trader=t, taa_params=taa_param)
+    taa_param = TAAParam()  # type in here
+    fa_param = FAParam() # type in here
+    b = FundBacktestEngine(data_manager=m, trader=t, taa_params=taa_param, fa_params=fa_param)
     b.init()
     b.run(saa=saa)
 
 def fund_realtime(m: FundDataManager, saa: AssetWeight):
-    taa_param = TAAParam()  # type in here
     asset_param = AssetTradeParam() # type in here
     fund_param = FundTradeParam() # type in here
     t = FundTrader(asset_param, fund_param)
-    b = FundEngine(data_manager=m, trader=t, taa_params=taa_param)
+    taa_param = TAAParam()  # type in here
+    fa_param = FAParam() # type in here
+    b = FundEngine(data_manager=m, trader=t, taa_params=taa_param, fa_params=fa_param)
     b.init()
     b.setup(saa)
     _dts = m.get_trading_days()
@@ -214,13 +239,27 @@ def fund_realtime(m: FundDataManager, saa: AssetWeight):
     cur_asset_position = AssetPosition(cash=cash)
     cur_fund_position = FundPosition(cash=cash)
 
-    trade_list = b.calc_fund_trade(calc_dt, cur_fund_position, cur_asset_position)
+    b.prep_data(calc_dt)
+    trade_list = b.calc_trade(calc_dt, cur_asset_position, cur_fund_position)
+    b.prep_data(final_dt)
+    traded_list = b.finalize_trade(final_dt, trade_list, cur_asset_position, cur_fund_position)
     print(trade_list)
-    traded_list = b.finalize_fund_trade(final_dt, trade_list, cur_fund_position, cur_asset_position)
     print(traded_list)
     print(cur_fund_position)
     print(cur_asset_position)
 
+def taa_result(m: FundDataManager):
+    start_date=datetime.date(2010,1,1)
+    end_date=datetime.date(2020,5,20)
+    index_id = 'hs300'
+    taaParam = TAAParam(HighThreshold = 1,
+                        HighStop = 0.45,
+                        HighMinus = 0.07,
+                        LowStop = 0.43,
+                        LowThreshold = 0.15,
+                        LowPlus = 0.06) 
+    return ReportHelper.get_taa_result(index_id=index_id, start_date=start_date, end_date=end_date, taaParam=taaParam, dm=m)
+
 def test():
     from ...data.manager.score import FundScoreManager
     m = FundDataManager('20150101', '20160101', score_manager=FundScoreManager())
